<?php
if(!defined("WHMCS")) die("This file cannot be accessed directly");

class WAPI extends WRequest
{
	private static $instance;
	protected $db;
	protected $config=array();
	protected $timeout=20;
	
	function __construct($db)
	{
		parent::__construct();
		$this->setDBconnection($db);
		if(function_exists('mb_internal_encoding')) mb_internal_encoding("UTF-8");
	}
	
	function __destruct()
	{
		
	}
	
	public static function getInstance($db=NULL)
	{
		if(!self::$instance) self::$instance = new self($db);
		return self::$instance;
	}
	
	public function getConfig($force=false)
	{
		static $config=array();
		if(empty($config) || $force)
		{
			$config=$this->db->getArray('SELECT * FROM `mod_weberp_config`;','cvalue','cname');
			$config['payments']=json_decode($config['payments'],true);
		}
		return $config;
	}
	
	#Get Addon Config
	public function getModuleParams($key=null)
	{
		static $modConfig=array();
		if(empty($modConfig))
		{
			$modConfig=$this->db->getArray('SELECT `setting`,`value` FROM `tbladdonmodules` WHERE module=`weberp`;','value','setting');
		}
		if(!empty($key)) return isset($modConfig[$key])?$modConfig[$key]:'';
		return $modConfig;
	}
	
	public function getHost()
	{
		if(isset($_SERVER['SERVER_NAME']) && $_SERVER['SERVER_NAME']) return $_SERVER['SERVER_NAME'];
		
		$hostname=$this->db->getValue('SELECT `value` FROM `tblconfiguration` WHERE `setting`="SystemURL";');
		return parse_url($hostname, PHP_URL_HOST);
	}
	
	public function getTaxType()
	{
		static $type;
		if(empty($type)) $type=$this->db->getValue('SELECT `value` FROM `tblconfiguration` WHERE `setting`="taxtype";');
		return $type;
	}

	public function getCustomfieldValue($fieldID,$userID)
	{
		static $values=array();
		$key=$fieldID.$userID;
		if(isset($values[$key])) return $values[$key];
		
		$values[$key]=$this->db->getValue('SELECT `value` FROM `tblcustomfieldsvalues` WHERE `relid`='.$this->db->quoteValue($userID).' AND `fieldid`='.$this->db->quoteValue($fieldID).';');
		return $values[$key];
	}
			
	public function dateUnix($date='now')
	{
		return is_numeric($date)?$date:strtotime(str_replace('/','-',$date));
	}
	
	public function dateFormat($date='now',$format='d-m-Y')
	{
		return date($format,$this->dateUnix($date));
	}
	
	public function dateSQL($date='now')
	{
		return $this->dateFormat($date,'Y-m-d');
	}
	
	public function datetimeSQL($date='now')
	{
		return $this->dateFormat($date,'Y-m-d H:i:s');
	}
	
	public function dateFix($value,$default=NULL,$format='Y-m-d')
	{
		return $this->isDateValid($value)?$this->dateFormat($value,$format):$default;
	}
	
	public function dateGMT($date='now',$format='Y-m-d H:i:s',$tz=null)
	{
		$timezone = $tz?$tz:date_default_timezone_get();
		$myDateTime = new DateTime($date, new DateTimeZone($timezone));
		$myDateTime->setTimezone(new DateTimeZone('GMT'));
		return $myDateTime->format($format);
	}
	
	public function dateLocal($date='now',$format='Y-m-d H:i:s')
	{
		$timezone = date_default_timezone_get();
		$myDateTime = new DateTime($date, new DateTimeZone('GMT'));
		$myDateTime->setTimezone(new DateTimeZone($timezone));
		return $myDateTime->format($format);
	}
	
	public function isDateValid($value)
	{
		if(!is_string($value)) return false;
		$value=str_replace('/','-',trim($value));
		return (empty($value) || substr($value,0,4)=='-000' || in_array($value,array('1970-01-01','00:00:00','0000-00-00 00:00:00','0000-00-00')) || strtotime($value)===false)?false:true;
	}
	
	function printJSON($data=array(),$options=0)
	{
		$this->setHeader('Content-Type','application/json; charset=utf-8',true);
		$this->db->close();
		http_response_code(200);
		echo json_encode($data,$options);
		exit();
	}
		
	function setHeader($name,$val=null,$replace=true)
	{
		$header=is_null($val)?$name:$name.': '.$val;
		@header($header,$replace);
	}
	
	public function getMessages($clear=true)
	{
		$html='';
		$quequeMsgs=$this->getArray('messages',array(),'SESSION');
		if(count($quequeMsgs))
		{
			$html='<div id="system-messages" class="messages">';
			foreach($quequeMsgs as $type=>$messages)
			{
				if($type=='error') $type='danger';
				foreach($messages as $msg)
				{
					$html.='<div class="alert alert-'.$type.'">'.$this->getLang($msg).'</div>';
				}
			}
			$html.='</div>';
			if($clear) $this->unsetVar('messages','SESSION');
		}
		return $html;
	}
	
	public function setMessage($msg='',$type='info')
	{
		$msg=is_array($msg)?print_r($msg,true):trim($msg); //temp
		if(!empty($msg))
		{
			$quequeMsgs=$this->getArray('messages',array(),'SESSION');
			$quequeMsgs[$type][$msg]=$msg;
			$this->setVar('messages',$quequeMsgs,'SESSION');
		}
	}
	
	public function redirect(string $url,$msg='',$type='info',$seconds=0)
	{
		$this->setMessage($msg,$type);

		$url=preg_replace('/(?|&)_t=([0-9]+)/','',urldecode($url));
		$sep=strpos($url,'?')===false?'?':'&';
		$url.=$sep.'_t='.time();
		
		if(headers_sent() || $seconds>0)
		{
			echo '<script>setTimeout(function(){window.location.href='.json_encode($url).';},'.($seconds*1000).');</script>';
		}
		else
		{
			$this->setHeader('Location',$url);
		}
		exit();
	}
	
	function fixPath($path,$sep='')
	{
		if(empty($sep)) $sep=DIRECTORY_SEPARATOR;
		$path=str_replace(array("\\","/"),$sep,$path);
		return rtrim($path,DIRECTORY_SEPARATOR);
	}
	
	function rename($from,$to)
	{
		$from=$this->fixPath($from);
		return $this->file_exists($from) && rename($from,$this->fixPath($to));
	}
		
	public function getFolders($folderPath)
	{
		if(!is_dir($folderPath)) return array();
		
		$folders=array();
		foreach (new DirectoryIterator($folderPath) as $file)
		{
			if($file->isDot()) continue;
			if($file->isDir()) $folders[]=$file->getFilename();
		}
		return $folders;
	}
	
	public function getFiles($folderPath,$ext=array(),$fullPath=false)
	{		
		$folderPath=rtrim($folderPath,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR;
		if(!is_dir($folderPath)) return array();
		
		$files=array();
		$results = scandir($folderPath);
		foreach($results as $file)
		{
			if ($file === '.' || $file === '..' || !is_file($folderPath.$file)) continue;
			$extension = pathinfo($file, PATHINFO_EXTENSION);
			if(count($ext)>0 && !in_array(".".$extension,$ext)) continue;
			$files[]=$fullPath? $folderPath.$file : $file;
		}
		return $files;
	}
	
	///Delete Folder with all files
	//Returns true|false
	public function deleteFolder($folderPath,$recursive=false)
	{
		$folderPath=rtrim($folderPath,DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR;
		if(!is_dir($folderPath)) return false;
		
		$files =$this->getFiles($folderPath,array(),true);
		if(count($files))
		{
			foreach($files as $file)
			{
				$this->deleteFile($file); // delete file
			}
		}
		
		if($recursive)
		{
			$folders=$this->getFolders($folderPath);
			if(count($folders))
			{
				foreach($folders as $folder)
				{
					$this->deleteFolder($folderPath.'/'.$folder,true);
				}
			}
		}
		
		return rmdir($folderPath);
	}
	
	function deleteFile($file)
	{
		if($this->file_exists($file)) return unlink($file);
		return false;
	}
	
	function writeFile($file,$data,$append=false)
	{
		return file_put_contents($file,$data,$append?(FILE_APPEND | LOCK_EX): LOCK_EX);
	}
	
	function readFile($file)
	{
		if(!$this->file_exists($file)) return '';
		return file_get_contents($file);
	}
	
	function file_exists($file)
	{
		return !empty($file) && is_file($file) && file_exists($file);
	}
	
	function folder_exists($dir)
	{
		return !empty($dir) && is_dir($dir) && file_exists($dir);
	}
	
	public function createFolders($folderPath)
	{
		if($this->folder_exists($folderPath)) return true;
		$folderPath=str_replace(WEBERP_ROOT,'',$this->fixPath($folderPath,DIRECTORY_SEPARATOR));
		$folders=explode(DIRECTORY_SEPARATOR,$folderPath);
		$folders=array_filter($folders);
		
		$folderPath=WEBERP_ROOT;
		foreach($folders as $folder)
		{
			if(strpos($folder,':'))
			{
				$folderPath=$folder;
			}
			else
			{
				$folderPath.=DIRECTORY_SEPARATOR.$folder;
				if(!$this->folder_exists($folderPath)) mkdir($folderPath);
			}
		}
		return true;
	}
	
	function downloadFile($url, $filepath,$resource=NULL)
	{
		$user_agent=(!empty($_SERVER['HTTP_USER_AGENT']))?$_SERVER['HTTP_USER_AGENT']:'Mozilla/5.0 (WebERP v1.0)';
		$ch=$resource===NULL?curl_init():$resource;
		 
		$this->createFolders(dirname($filepath));
		 
		$fp = fopen($filepath, 'w+');
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_USERAGENT,$user_agent);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, false);
		curl_setopt($ch, CURLOPT_BINARYTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false );
		curl_setopt($ch, CURLOPT_ENCODING, '' );
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
		curl_setopt($ch, CURLOPT_TIMEOUT, 90);
		curl_setopt($ch, CURLOPT_FILE, $fp);
		curl_exec($ch);
		$code=curl_getinfo($ch,CURLINFO_HTTP_CODE);
		if($resource===NULL) curl_close($ch);
		fclose($fp);
		 
		$status=$this->in_range($code,200,201) && $this->file_exists($filepath) && @filesize($filepath)>2? true : false;
		if(!$status) $this->deleteFile($filepath);
		 
		return $status;
	}
	
	function print_data($values,$ret=false)
	{
		if(!defined('CLI'))
			$data='<pre>'.htmlspecialchars(print_r($values,true)).'</pre>';
		else
			$data=print_r($values,true)."\n";
		
		if($ret) return $data;
		echo $data;
	}
	
	public function dump($var)
	{
		ob_start();
		var_dump($var);
		return ob_get_clean();
	}
	
	function JSONdecode($rawData,$toArray=true)
	{
		$sep="<br />\n{";
		if(substr($rawData,0,1)!='{' && strpos($rawData,$sep)!==false)
		{
			$listData=explode($sep,$rawData,2);
			$rawData='{'.$listData[1];
		}
		return @json_decode($rawData,$toArray);
	}
	
	function in_range($number, $min, $max, $inclusive = true)
	{
		$number=$this->toFloat($number);
		$min=$this->toFloat($min);
		$max=$this->toFloat($max);

		if($max>$min)
		{
			return $inclusive
				? ($number >= $min && $number <= $max)
				: ($number > $min && $number < $max) ;
		}
		return FALSE;
	}
	
	function getLang($key,$force=false,$language='')
	{
		global $_LANG,$_ADDONLANG;
		static $langs;
		if(empty($langs)) $force=true;
		if(empty($_ADDONLANG)) $_ADDONLANG=array();
		if(empty($_LANG)) $_LANG=array();

		if($force)
		{
			$default=$this->db->getValue('SELECT `value` FROM `tblconfiguration` WHERE `setting`="Language";');
			$lang=$language?$language:strtolower($this->getVar('Language',$default,'SESSION'));
			if($lang && $this->file_exists(WEBERP_ROOT.'/lang/'.$lang.'.php'))
			{
				$bkp=$_ADDONLANG;
				$_ADDONLANG=array();
				require (WEBERP_ROOT.'/lang/'.$lang.'.php');
				$_ADDONLANG=array_merge($bkp,$_ADDONLANG);
			}
			if($lang && $this->file_exists(WEBERP_ROOT.'/lang/override/'.$lang.'.php'))
			{
				$bkp=$_ADDONLANG;
				$_ADDONLANG=array();
				require (WEBERP_ROOT.'/lang/'.$lang.'.php');
				$_ADDONLANG=array_merge($bkp,$_ADDONLANG);
			}
		}
		
		$langs=array_merge($_LANG,$_ADDONLANG);		
		return isset($langs[$key])?$langs[$key]:$key;
	}
	
	public function fill($number,$char,$size=5,$pos='prefix')
	{
		$length=strlen($number);
		if($length>=$size) return $number;
		$extrachars='';
		for($i=0;$i<$size-$length;$i++) $extrachars.=$char;
		return ($pos=='prefix')?$extrachars.$number:$number.$extrachars;
	}
	
	function doDownload($file_path,$filename='')
	{
		if(empty($file_path)) exit('file_path is empty');
		if(!$this->file_exists($file_path)) exit('File not exists');
		
		@ini_set('error_reporting',0);
		@ini_set('display_errors',0);
		@ini_set('zlib.output_compression', 'Off');
		
		$path_parts = pathinfo($file_path);
		$file_name  = $path_parts['basename'];
		$file_ext   = $path_parts['extension'];
		$file_size  = filesize($file_path);
		
		if(!empty($filename))
		{
			$file_name=trim(str_replace(array('#','.'.$file_ext),'',$filename)).'.'.$file_ext;
		}
		
		// set the headers, prevent caching
		header('Content-Description: File Transfer',true);
		header('Cache-Control: private, max-age=0, no-cache, no-store, must-revalidate, post-check=0, pre-check=0',true);
		header('Pragma: public',true);
		header('Connection: Keep-Alive',true);
		header('Expires: Mon, 01 Jan 2010 00:00:00 GMT',true);
		header('X-Logged-In: True',true);
		header('Content-Disposition: attachment; filename="'.$file_name.'"',true);
		header('Content-Transfer-Encoding: binary',true);
		header("Content-Length: ".$file_size);
		
		$ctype_default = "application/octet-stream";
		$content_types = array(
			"exe" => "application/octet-stream",
			"zip" => "application/zip",
			"rar" => "application/x-rar-compressed",
			"mp3" => "audio/mpeg",
			"mpg" => "video/mpeg",
			"avi" => "video/x-msvideo",
			"pdf" => "application/pdf",
			"csv" => "text/csv",
			"txt" => "text/plain",
		);
		
		$ctype = isset($content_types[$file_ext]) ? $content_types[$file_ext] : $ctype_default;
		header("Content-Type: " . $ctype,true);
		http_response_code(200);
		ob_clean(); 
		flush();
		readfile($file_path);
		exit;
	}
	
	
	function invoiceFormat($row)
	{
		$config=$this->getConfig();
		$customfields=array('vatid'=>'vatidcustom','doy'=>'taxresidencefield','job'=>'jobfield','invoicetype'=>'invoicetypefield','invoiceto'=>'invoicetofield');
	
		$invoice=array();
		$invoice['id']=$row['id'];
		$invoice['invoicenum']=$row['invoicenum'];
		$invoice['status']=$row['status'];
		$invoice['date']=$row['status']=='Paid'?$row['datepaid']:$row['date'];
		$invoice['duedate']=$row['status']=='Unpaid'?$row['duedate']:NULL;
		$invoice['notes']=$row['notes'];
		$invoice['subtotal']=(float)$row['subtotal'];
		$invoice['tax']=(float)$row['tax'];
		$invoice['total']=(float)$row['total'];
		$invoice['credit']=(float)$row['credit'];
		$invoice['ispaid']=$row['status']=='Paid'?1:0;
		$invoice['gateway']=$row['paymentmethod'];
		$invoice['paymentmethod']=isset($config['payments'][$row['paymentmethod']])?(int)$config['payments'][$row['paymentmethod']]:0;
		$invoice['ignored']=0;
		
		if($invoice['total']==$invoice['credit'] && $config['ignorecredit']==2)
		{
			$invoice['ignored']=0;
		}

		$invoice['billing']=array(
			'fullname'=>$row['firstname'].' '.$row['lastname'],
			'companyname'=>$row['companyname'],
			'email'=>$row['email'],
			'address1'=>$row['address1'],
			'address2'=>$row['address2'],
			'city'=>$row['city'],
			'state'=>$row['state'],
			'zip'=>$row['postcode'],
			'country'=>$row['country'],
			'phone'=>str_replace(array(' ','-','+','.'),'',$row['phonenumber']),
			'language'=>$row['language'],
			'taxexempt'=>$row['taxexempt']=='on' || $row['taxexempt']==1?1:0,
		);
			
		if($config['vatid']=='native')
		{
			$invoice['billing']['vatid']=$row['tax_id'];
			unset($customfields['vatid']);
		}
			
		foreach($customfields as $keyField=>$configField)
		{
			$invoice['billing'][$keyField]=isset($config[$configField]) && $config[$configField]!=''?$this->getCustomfieldValue($config[$configField],$row['userid']):'';
		}
			
		$invoice['items']=array();
		$rsItems=$this->db->query('SELECT * FROM `tblinvoiceitems` WHERE `invoiceid`='.$this->db->quoteValue($row['id']).' ORDER BY `id` ASC');
		
		$taxType=$this->getTaxType();		
		while($item=$this->db->fetch_array($rsItems))
		{
			$total=$subtotal=(float)$item['amount'];
			$tax=0;
			if($subtotal && $row['taxrate'] && $item['taxed'] && $taxType=='Exclusive')
			{
				$total=round($subtotal+$subtotal*$row['taxrate']/100,2);
				$tax=round($total-$subtotal,2);
			}
			else if($subtotal && $row['taxrate'] && $item['taxed'] && $taxType=='Inclusive')
			{
				$subtotal = round($total/(1+($row['taxrate']/100)),2);
				$tax=round($total-$subtotal,2);
			}
			
			if($item['type']=='AddFunds' && $config['ignorecredit']==1)
			{
				$invoice['ignored']=1;
			}
			
			if($item['type']=='Invoice' && $config['ignorecredit']==1)
			{
				$invoice['ignored']=1;
			}
			
			$invoice['items'][]=array(
				'product_id'=>$item['id'],
				'sku'=>$item['type'],
				'relid'=>$item['relid'],
				'name'=>$item['description'],
				'unitprice'=>$item['amount'],
				'quantity'=>1,
				'taxrate'=>$row['taxrate'] && $item['taxed']?$row['taxrate']:0,
				'subtotal'=>$subtotal,
				'tax'=>$tax,
				'total'=>$total,
			);
		}
				
		if($invoice['ignored'])
		{
			$this->db->insertUpdate('mod_weberp_ignored',array('id'=>$row['id']));
		}
		
		$invoice['transactions']=array();
		$rsTrans=$this->db->query('SELECT * FROM `tblaccounts` WHERE `invoiceid`='.$this->db->quoteValue($row['id']).' ORDER BY `date` ASC');
		while($trans=$this->db->fetch_array($rsTrans))
		{
			$invoice['transactions'][]=array(
				'id'=>$trans['id'],
				'paymentmethod'=>isset($config['payments'][$trans['gateway']])?(int)$config['payments'][$trans['gateway']]:0,
				'gateway'=>$trans['gateway'],
				'created'=>$trans['date'],
				'amount'=>$trans['amountin']?$trans['amountin']:$trans['amountout'],
				'transid'=>$trans['transid'],
			);
		}
		
		return $invoice;
	}
	
}