<?php
if(!defined("WHMCS")) die("This file cannot be accessed directly");
class WDB
{
	private static $instance;
	protected $conn=null;
	protected $mysqlActive=true;
	public $counter=0;
	public $queries=array();
	protected $sql_query='';
	protected $useMysqli=true;
	protected $last_query=null;
	protected $enableErrors=0;
	public $forceMysqli=true;
	public $enableExplain=false;
	
	private $host='127.0.0.1';
	private $dbname=null;
	private $dbuser=null;
	private $dbpass=null;
	private $charset=null;
	private $logger=null;
	
	public function display_errors($mode)
	{	
		$this->enableErrors=(int)$mode;
	}
	
	function __construct()
	{
		//$this->connect();
	}
	
	function __destruct()
	{
		$this->close();
	}
	
	public function setConnection($host,$dbname,$dbuser,$dbpass=NULL,$charset=NULL)
	{
		$this->host=$host;
		$this->dbname=$dbname;
		$this->dbuser=$dbuser;
		$this->dbpass=$dbpass;
		$this->charset=$charset;
		$this->connect();
	}
	
	/*Get One Instance of Database*/
	public static function getInstance()
	{
		if(!self::$instance)
		{
			self::$instance = new self();
		}
		return self::$instance;
	}
	
	//Connect to database if no connection found
	public function connect()
	{	
		if($this->conn) return true;
		if(empty($this->host) || empty($this->dbname) || empty($this->dbuser)) return false;
		
		//Check if there is already connection
		if($this->forceMysqli || !function_exists('mysql_query') || !@mysql_query('SELECT 1'))
		{
			$this->mysqlActive=false;
		}
		
		//Check if Mysqli is available
		if(function_exists("mysqli_connect") && !$this->mysqlActive)
		{ 
			$this->useMysqli=true;
		}
		
		//Fix password issue
		if(empty($this->dbpass)) $this->dbpass=NULL;
		
		#Connect to DB
		if(!$this->mysqlActive)
		{
			if($this->useMysqli)	
				$this->conn = @mysqli_connect($this->host,$this->dbuser,$this->dbpass);
			else
				$this->conn = @mysql_connect($this->host,$this->dbuser,$this->dbpass);
			
			#Check Connect
			if(!$this->conn)
			{
				echo 'Could not connect: ';
				if($this->useMysqli)
					echo str_replace(array($this->dbpass,$this->dbuser),'**',mysqli_connect_error());
				else
					echo str_replace(array($this->dbpass,$this->dbuser),'**',mysql_error());
				die();
			}
			
			#SelectDB
			if($this->useMysqli){		
				@mysqli_select_db($this->conn,$this->dbname);
			}else{
				@mysql_select_db($this->dbname,$this->conn);
			}
			
			$err=$this->error();
			
			if(!empty($err))
			{
				$this->log("Select DB Error:". $err);
				exit;
			}
			#Charset
			if(!empty($this->charset)) $this->setCharset($this->charset);
		}
	}
	
	function logger(callable $callback)
	{
		if(is_callable($callback)) $this->logger=$callback;
	}
	
	function log($error='')
	{
		$error=empty($error)?'Error on Query: '.$this->sql_query."\n ".$this->error():$error;
		
		if(is_callable($this->logger))
		{
			call_user_func_array($this->logger,array('MySQL',$error."\n",'mysql'));
			//call_user_func_array($this->logger,array('MySQL',print_r(debug_backtrace(),true)."\n",'mysql'));
		}
		
		if($this->enableErrors)
		{
			throw new Exception($error);
			//echo '<div style="border:1px dotted red;">'.$error.'</div>';
			//echo debug_backtrace();
			//exit;
		}
	}
	
	public function setCharset($charset='utf8')
	{
		$this->charset=$charset;
		$result=$this->useMysqli?mysqli_set_charset($this->conn,$this->charset):@mysql_set_charset($this->charset,$this->conn);
		if(!$result) $this->log("Error loading character set ".$this->charset.": ".$this->error());
		return $result;
	}
	
	
	//Get Connection
	public function getConnection()
	{
		return $this->conn;
	}
	
	//Escape String
	public function safe($q)
	{
		if($this->useMysqli) 
			return mysqli_real_escape_string($this->conn,$q);
		else 
			return @mysql_real_escape_string($q);
	}
	
	//Quote Field Name
	public function quoteField(string $n)
	{
		$n=str_replace('`','',$n);
	
		$prefix='';
		$postfix='';
		
		if(preg_match('/([a-zA-Z]+)\((.*)\)$/',$n,$match))
		{
			$prefix=$match[1].'(';
			$postfix=')';
			$n=$match[2];
		}
		
		$field=$this->safe($n);
		if(strpos($field,'.')!==false)
		{
			list($fpre,$field)=explode('.',$n,2);
			$prefix.=$fpre.'.';
		}		
		return $prefix.'`'.$this->safe($field).'`'.$postfix;
	}
	
	//Quote Key Name
	public function quoteValue($n)
	{
		if(is_array($n)) return $this->quoteArray($n);
		return '"'.$this->safe($n).'"';
	}
	
	public function quoteArray(array $arr)
	{
		if(empty($arr)) return '';
		return implode(',',array_map(array($this,'quoteValue'),$arr));
	}
	
	//Get columns
	public function getTables()
	{
		$tables=array();
		if(!empty($tables)) return $tables;
		
		$rs = $this->query('SHOW TABLES FROM '.$this->quoteField($this->dbname));
		$tables=array();
		while ($row = $this->fetch_array($rs,'MYSQL_NUM'))
		{
			$tables[]=$row[0];
		}
		return $tables;
	}
		
	//Get columns
	public function getColumns($table)
	{
		$columns=array();
		if(empty($table) || !empty($columns)) return $columns;
		
		$rs = $this->query('SHOW COLUMNS FROM '.$this->quoteField($table).';');
		while ($row = $this->fetch_array($rs))
		{
			$columns[$row['Field']]=$row;
		}
		return $columns;
	}
	
	//Set query
	public function query($q)
	{
		$this->sql_query=trim($q);
		
		$startTime=microtime(true);
		if($this->useMysqli){ 
			$this->last_query=mysqli_query($this->conn,$this->sql_query);
		}else{
			$this->last_query=@mysql_query($this->sql_query);
		}
		
		$log=array('time'=>number_format(round(microtime(true) - $startTime,3),3),'sql'=>$this->sql_query);
		if($this->enableExplain && substr($this->sql_query,0,6)=='SELECT')
		{
			$log['explain']=$this->explain($this->sql_query);
		}
		$this->queries[]=$log;
		
		if($this->errno($this->last_query)) $this->log();
		$this->counter++;	
		return $this->last_query;
	}
	
	public function explain($q)
	{
		$backup=$this->last_query;
		$this->enableExplain=false;
		
		$explainData=$this->getArray("EXPLAIN ".$q);
		
		$this->last_query=$backup;
		$this->enableExplain=true;
		return $explainData;
	}
	
	private function getValueStatement($value)
	{
		if(is_array($value))
		{
			$data=array_map(array($this, 'safe'), $value);
			return ' IN ('.implode(',',$data).')';
		}
		else
		{
			$unescaped=array('CURRENT_TIMESTAMP','NOW()');
			$escValue=!in_array(strtoupper($value),$unescaped)?$this->quoteValue($value):$value;
			return $this->isNULL($value)?' is NULL':' = '.$escValue;
		}
	}
	
	private function buildWhere($where)
	{
		if(is_array($where))
		{
			$wh=array();
			foreach($where as $key=>$value) $wh[]=$this->quoteField($key).$this->getValueStatement($value);
			$wh=@implode(' AND ',$wh);
		}
		else
		{
			$wh=(string)$where;
		}
		return $wh;
	}
	
	//Build SQL
	public function buildSQL($table,$select='*',$where=array(),$orderby='',$limit='')
	{
		if(empty($table)) return false;
		$wh=$this->buildWhere($where);
		
		if(is_array($select)) $select=implode(', ',$select);
		if(!empty($wh)) $wh=' WHERE '.$wh;
		return trim('SELECT '.$select.' FROM '.$this->quoteField($table).$wh.' '.$orderby.' '.$limit).';';
	}
	
	
	//Select Row
	public function select($table,$select='*',$where=array(),$orderby='',$limit='')
	{
		return $this->query($this->buildSQL($table,$select,$where,$orderby,$limit));
	}
		
	//Delete Row
	public function delete($table,$where=array())
	{
		if(empty($table)) return false;
		$wh=$this->buildWhere($where);
		//if(empty($wh)) return false;
		return $this->query('DELETE FROM '.$this->quoteField($table).' WHERE '.$wh.';');
	}
	
	//Insert Row
	public function insert($table,$fields=array())
	{
		if(empty($table) || !count($fields)) return false;
		$values=$columns=array();
		foreach($fields as $key=>$value)
		{
			$columns[]=$this->quoteField($key);
			$values[]=$this->isNULL($value)?'NULL':$this->quoteValue($value);
		}
		return $this->query('INSERT IGNORE INTO `'.$table.'`('.implode(',',$columns).') VALUES('.implode(',',$values).');');
	}
	
	public function insertUpdate($table,$fields=array())
	{
		if(empty($table) || !count($fields)) return false;
		$update=$values=$columns=array();
		foreach($fields as $key=>$value)
		{
			$qKey=$this->quoteField($key);
			$qVal=$this->isNULL($value)?'NULL':$this->quoteValue($value);
			
			$columns[]=$qKey;
			$values[]=$qVal;
			$update[]=$qKey.'='.$qVal;			
		}
		return $this->query('INSERT INTO `'.$table.'`('.implode(',',$columns).') VALUES('.implode(',',$values).') ON DUPLICATE KEY UPDATE '.implode(',',$update).';');
	}
	
	public function isNULL($value)
	{
		return (!is_numeric($value) && ($value===NULL || strtoupper($value)=='NULL'))?true:false;
	}
	
	//Update Row
	public function update($table,$fields=array(),$where=array())
	{
		if(empty($table) || !count($fields)) return false;
		$set=array();

		foreach($fields as $key=>$value)
		{
			$newValue=$this->isNULL($value)?'NULL': $this->quoteValue($value);
			$set[]=$this->quoteField($key).'='.$newValue;
		}
		
		$wh=$this->buildWhere($where);
		//if(empty($wh)) return false;
		return $this->query('UPDATE `'.$table.'` SET '.implode(', ',$set).' WHERE '.$wh.';');
	}
	
	function insertUpdateID($table,$fields=array(),$id,$key='id')
	{
		if($id)
		{
			$this->update($table,$fields,array($key=>$id));
			return $id;
		}
		else
		{
			$this->insert($table,$fields);
			return $this->insert_id();
		}
	}
	
	function updateHit($table,$col,$where=array(),$increase=1)
	{
		if(empty($table) || empty($col) || $increase==0) return false;
		$wh=$this->buildWhere($where);
		
		if($increase>0) $increase='+'.$increase;
		return $this->query('UPDATE `'.$table.'` SET '.$this->quoteField($col).'=('.$this->quoteField($col).' '.$increase.') WHERE '.$wh.';');	
	}
	
	//Get query
	public function getSQL()
	{
		return $this->sql_query;
	}
	
	//Single column
	public function getValue($q)
	{		
		$data=$this->fetch_array($this->query($q),'MYSQL_NUM');
		return isset($data[0])?$data[0]:null;
	}
	
	//Get Single Row
	public function getRow($q)
	{
		return (array)$this->fetch_array($this->query($q));
	}
	
	//Get Array of rows
	public function getArray($q,$key=NULL,$id=NULL)
	{
		$return=array();
		$rs=$this->query($q);
		while($r=$this->fetch_array($rs))
		{
			$value=empty($key)?$r:$r[$key];
			if($id===NULL) 
				$return[]=$value;
			else 
				$return[$r[$id]]=$value;
		}
		return $return;
	}
	
	public function fetch_array($rs=null,$type ='MYSQL_ASSOC')
	{
		if(is_null($rs)) $rs=$this->last_query;
		$type=strtoupper($type);
		if($this->useMysqli)
		{
			$type=str_replace('MYSQL','MYSQLI',$type);
		}
		if($rs === FALSE) return false;
		if($this->useMysqli)
			return mysqli_fetch_array($rs,constant($type));
		else
			return @mysql_fetch_array($rs,constant($type));
	}
	
	//Fetch row
	public function fetch_row($rs=null)
	{
		if(is_null($rs)) $rs=$this->last_query;
		if($this->useMysqli)
			return mysqli_fetch_row($rs);
		else
			return @mysql_fetch_row($rs);
	}
	
	//get last insert id
	public function insert_id($rs=null)
	{
		if(is_null($rs)) $rs=$this->last_query;
		if($this->useMysqli)
			return mysqli_insert_id($this->conn);
		else
			return @mysql_insert_id();
	}
	
	//get number of fields
	public function num_fields($rs=null)
	{
		if(is_null($rs)) $rs=$this->last_query;
		if($this->useMysqli)
			return (int)mysqli_num_fields($rs);
		else
			return (int)@mysql_num_fields($rs);
	}	
	
	//get number of rows
	public function num_rows($rs=null)
	{
		if(is_null($rs)) $rs=$this->last_query;
		if($this->useMysqli)
			return (int)mysqli_num_rows($rs);
		else
			return (int)@mysql_num_rows($rs);
	}
	
	//get affected rows (insert/update/delete)
	public function affected_rows()
	{
		if($this->useMysqli) return (int)@mysqli_affected_rows($this->conn);
		return (int)@mysql_affected_rows($this->conn);
	}
	
	//Get error message
	public function error($rs=null)
	{
		if(is_bool($this->conn)) return 'MySQL Error';
		
		if($this->useMysqli)
			return mysqli_error($this->conn);
		else
			return @mysql_error($this->conn);
	}
	
	//Get error number
	public function errno($rs=null)
	{
		if($this->useMysqli)
			return mysqli_errno($this->conn);
		else
			return @mysql_errno($this->conn);
	}
	
	
	public function isMysql()
	{
		if(is_null($this->conn)) return false;
		$res_type = is_resource($this->conn) ? get_resource_type($this->conn) : gettype($this->conn);
		if(strpos($res_type, 'mysql') !== false) return true;
		return false;
	}
	
	public function ping()
	{
		if($this->useMysqli)
			return @mysqli_ping($this->conn);
		else
			return @mysql_ping($this->conn);
	}
	
	//close connection
	public function close()
	{
		if(!is_null($this->conn) && $this->isMysql() && $this->ping())
		{
			if($this->useMysqli)
				@mysqli_close($this->conn);
			else
				@mysql_close($this->conn);
		}
		$this->conn=null;
	}
	
	/*
		Import SQL data
	*/
	public function source_query($dbms_schema)
	{
		@set_time_limit(0);
		//@ini_set('memory_limit', '10000M');

		$sql_query = @fread(@fopen($dbms_schema, 'r'), @filesize($dbms_schema)) or exit('problem with '.$dbms_schema);
		$sql_query = $this->remove_remarks($sql_query);
		$sql_query = $this->split_sql_file($sql_query, ';');
		
		foreach($sql_query as $sql)
		{
			$this->query($sql);
		}
	}
	
	protected function remove_comments(&$output)
	{
	   $lines = explode("\n", $output);
	   $output = "";

	   // try to keep mem. use down
	   $linecount = count($lines);

	   $in_comment = false;
	   for($i = 0; $i < $linecount; $i++)
	   {
		  if( preg_match("/^\/\*/", preg_quote($lines[$i])) )
		  {
			 $in_comment = true;
		  }

		  if( !$in_comment )
		  {
			 $output .= $lines[$i] . "\n";
		  }

		  if( preg_match("/\*\/$/", preg_quote($lines[$i])) )
		  {
			 $in_comment = false;
		  }
	   }

	   unset($lines);
	   return $output;
	}

	//
	// remove_remarks will strip the sql comment lines out of an uploaded sql file
	//
	protected function remove_remarks($sql)
	{
	   $lines = explode("\n", $sql);

	   // try to keep mem. use down
	   $sql = "";

	   $linecount = count($lines);
	   $output = "";

	   for ($i = 0; $i < $linecount; $i++)
	   {
		  if (($i != ($linecount - 1)) || (strlen($lines[$i]) > 0))
		  {
			 if (isset($lines[$i][0]) && $lines[$i][0] != "#")
			 {
				$output .= $lines[$i] . "\n";
			 }
			 else
			 {
				$output .= "\n";
			 }
			 // Trading a bit of speed for lower mem. use here.
			 $lines[$i] = "";
		  }
	   }

	   return $output;

	}

	//
	// split_sql_file will split an uploaded sql file into single sql statements.
	// Note: expects trim() to have already been run on $sql.
	//
	protected function split_sql_file($sql, $delimiter=';')
	{
	   // Split up our string into "possible" SQL statements.
	   $tokens = explode($delimiter, $sql);

	   // try to save mem.
	   $sql = "";
	   $output = array();

	   // we don't actually care about the matches preg gives us.
	   $matches = array();

	   // this is faster than calling count($oktens) every time thru the loop.
	   $token_count = count($tokens);
	   for ($i = 0; $i < $token_count; $i++)
	   {
		  // Don't wanna add an empty string as the last thing in the array.
		  if (($i != ($token_count - 1)) || (strlen($tokens[$i] > 0)))
		  {
			 // This is the total number of single quotes in the token.
			 $total_quotes = preg_match_all("/'/", $tokens[$i], $matches);
			 // Counts single quotes that are preceded by an odd number of backslashes,
			 // which means they're escaped quotes.
			 $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$i], $matches);

			 $unescaped_quotes = $total_quotes - $escaped_quotes;

			 // If the number of unescaped quotes is even, then the delimiter did NOT occur inside a string literal.
			 if (($unescaped_quotes % 2) == 0)
			 {
				// It's a complete sql statement.
				$output[] = $tokens[$i];
				// save memory.
				$tokens[$i] = "";
			 }
			 else
			 {
				// incomplete sql statement. keep adding tokens until we have a complete one.
				// $temp will hold what we have so far.
				$temp = $tokens[$i] . $delimiter;
				// save memory..
				$tokens[$i] = "";

				// Do we have a complete statement yet?
				$complete_stmt = false;

				for ($j = $i + 1; (!$complete_stmt && ($j < $token_count)); $j++)
				{
				   // This is the total number of single quotes in the token.
				   $total_quotes = preg_match_all("/'/", $tokens[$j], $matches);
				   // Counts single quotes that are preceded by an odd number of backslashes,
				   // which means they're escaped quotes.
				   $escaped_quotes = preg_match_all("/(?<!\\\\)(\\\\\\\\)*\\\\'/", $tokens[$j], $matches);

				   $unescaped_quotes = $total_quotes - $escaped_quotes;

				   if (($unescaped_quotes % 2) == 1)
				   {
					  // odd number of unescaped quotes. In combination with the previous incomplete
					  // statement(s), we now have a complete statement. (2 odds always make an even)
					  $output[] = $temp . $tokens[$j];

					  // save memory.
					  $tokens[$j] = "";
					  $temp = "";

					  // exit the loop.
					  $complete_stmt = true;
					  // make sure the outer loop continues at the right point.
					  $i = $j;
				   }
				   else
				   {
					  // even number of unescaped quotes. We still don't have a complete statement.
					  // (1 odd and 1 even always make an odd)
					  $temp .= $tokens[$j] . $delimiter;
					  // save memory.
					  $tokens[$j] = "";
				   }

				} // for..
			 } // else
		  }
	   }

	   return $output;
	}
}
class Query
{
	protected $where=array();
	protected $indexes=array();
	protected $db;
	
	function __construct(WDB $db)
	{
		$this->db=$db;
		if(!$this->db) throw new Exception('Database must be set');
	}
	
	function set($name,$value,$op='=',$oposite=false)
	{
		$opOp=$this->operator($op);
		if($opOp===false) throw new Exception('Invalid Operator '.$op);
		$this->where[]=array($name,$oposite?$opOp:trim($op),$value);
		
		end($this->where);
		$last_idx=key($this->where);
		$this->indexes[$name]=$last_idx;
		return true;
	}
	
	function remove($name)
	{
		$idx=isset($this->indexes[$name])?$this->indexes[$name]:false;
		if($idx!==false)
		{
			unset($this->where[$idx]);
			unset($this->indexes[$name]);
		}
	}
	
	function reset()
	{
		$this->where=array();
	}
	
	function operator($op)
	{
		$op=strtoupper(trim($op));
		
		$ops=array(
			'='=>'!=',
			'!='=>'=',
			'IS'=>'IS NOT',
			'IS NOT'=>'IS',
			'IN'=>'NOT IN',
			'NOT IN'=>'IN',	
			'>'=>'<',
			'<'=>'>',
			'>='=>'<=',
			'<='=>'>=',
			'LIKE'=>'NOT LIKE',
			'NOT LIKE'=>'LIKE',
		);
		return isset($ops[$op])?$ops[$op]:false;
	}
	
	function toSQL($sep='AND')
	{
		$q=array();
		foreach($this->where as $w)
		{
			$name=$this->db->quoteField($w[0]);
			$op=$w[1];
			$value=$w[2];
			
			if(is_object($value) && get_class($value)=='Query')
			{
				$value='('.(string)$value.')';
			}
			else if(is_array($value))
			{
				$data=array_map(array($this->db, 'safe'), $value);
				$value='('.implode(',',$data).')';
				if($op=='=') $op='IN';
				else if($op=='!=') $op='NOT IN';
			}
			else
			{
				$isNULL=$this->db->isNULL($value);
				$value=$isNULL?'NULL':$this->db->quoteValue($value);
				if($isNULL)
				{
					if($op=='=') $op='IS';
					else if($op=='!=') $op='IS NOT';
				}
			}
			$q[]=$name.' '.$op.' '.$value;
		}
		
		$sep=strtoupper($sep)=='AND'?'AND':'OR';
		return implode(' '.$sep.' ',$q);
	}
	
	public function __toString()
	{
		return $this->toSQL();
	}	
}