<?php
if(!defined("WHMCS")) die("This file cannot be accessed directly");

class WFormField extends Wbase
{
	private static $instance;
	protected $disabled=0;
	protected $api;
	
	function __construct($db=NULL)
	{
		parent::__construct();
		$this->setDBconnection($db);
		if(is_object($db)) $this->api=WAPI::getInstance($db);
	}
	
	public static function getInstance($db=NULL)
	{
		if(!self::$instance) self::$instance = new self($db);
		return self::$instance;
	}
	
	function setDisabled($m)
	{
		$this->disabled=(int)$m;
	}
	
	protected function parseAttributes($name,$attrs=array())
	{
		$id=str_replace(array('[',']'),'',$name);
		if(!is_array($attrs)) $attrs=array($attrs);
		if(!isset($attrs['id'])) $attrs['id']=$id;
		if(!isset($attrs['class'])) $attrs['class']='form-control';
		
		$atrstr=array();
		foreach($attrs as $aname=>$val)
		{
			if(!is_numeric($aname) && ($val=='' || is_null($val)) || $val===false) continue;
			if($val && ($aname=='readonly' || $aname=='disabled')) $val=$aname;
			$atrstr[]=is_numeric($aname)?trim($val):$aname.'="'.trim($val).'"';
		}
		return implode(' ',$atrstr);
	}
	
	public function button($name,$type='button',$attrs=array())
	{
		$onclick=isset($attrs['url']) && $attrs['url']?' onclick="location.href=\''.$attrs['url'].'\';"':'';
		if(isset($attrs['url'])) unset($attrs['url']);
		$atrstr=$this->parseAttributes($name,array_merge(array('class'=>'btn'),$attrs));
		return '<button'.$onclick.' type="'.$type.'" '.$atrstr.'>'.$this->api->getLang($name).'</button>';
	}
	
	public function saveButton($name,$class,$attrs=array())
	{
		$attrs['class']=rtrim('btn btn-sm btn-success '.$class);
		return $this->button($name,'submit',$attrs);
	}

	public function cancelButton($name,$class,$url='',$attrs=array())
	{
		$attrs['class']=rtrim('btn btn-sm btn-danger '.$class);
		if($url) $attrs['url']=$url;
		return $this->button($name,'button',$attrs);
	}
	
	public function newButton($name,$class,$url='',$id='',$attrs=array())
	{
		$attrs['class']=rtrim('btn-sm btn-success '.$class);
		if($url) $attrs['url']=$url;
		if($id) $attrs['id']=$id;
		return $this->button($name,'button',$attrs);
	}
	
	function checkbox($name,$value=1,$checked=false,$attrs=array(),$bootstrap=true)
	{
		//if($bootstrap) $this->addHead('js',TEMPLATE.'/js/plugins/bootstrap-switch.js');
		$ontext=isset($attrs['ontext'])?' '.$attrs['ontext']:'';
		$offtext=isset($attrs['offtext'])?' '.$attrs['offtext']:'';
		
		$atrstr=$this->parseAttributes($name,array_merge(array('class'=>''),$attrs));
		$html='<input name="'.$name.'" '.$atrstr.' value="'.$value.'" type="checkbox"'.($checked?' checked="checked"':'');
		$html.=!$bootstrap?' />': ' data-toggle="switch" data-on-color="info" data-off-color="info" data-on-text="'. htmlentities('<i class="fa fa-check"></i>', ENT_COMPAT|ENT_QUOTES,'UTF-8', true).$ontext.'" data-off-text="'.htmlentities('<i class="fa fa-times"></i>', ENT_COMPAT|ENT_QUOTES,'UTF-8', true).$offtext.'" /><span class="toggle"></span>';
		
		return $html;
	}
	
	function checkboxes($name,$options=array(),$selected=array(),$attrs=array(),$bootstrap=true)
	{
		$selected=is_array($selected)?$selected:array($selected);
		if(isset($attrs['bootstrapoff']) && $attrs['bootstrapoff'])
		{
			$bootstrap=false;
			unset($attrs['bootstrapoff']);
		}
		
		$html='';
		foreach($options as $option=>$valName)
		{
			$html.='<label>'.$this->checkbox($name,$option,in_array($option,$selected),$attrs,$bootstrap).' '.$this->api->getLang($valName).'</label> ';
		}
		return $html;
	}
	
	function select($name,$options=array(),$selected=array(),$attrs=array())
	{
		$attrs=empty($attrs)?array():$attrs;
		$html='';
		if($this->disabled)
		{
			$this->disabled=false;
			$html.=$this->select($name,$options,$selected,array('readonly'=>true,'class'=>'hide'));
			$this->disabled=true;
			$attrs['disabled']='disabled';
			$name.='-off';
		}
		
		$atrstr=$this->parseAttributes($name,$attrs);
		$selected=is_array($selected)?$selected:array($selected);
		$html.='<select name="'.$name.'" '.$atrstr.'>'."\n";
		foreach($options as $key=>$opname)
		{
			if(is_array($opname))
			{
				$html.='<optgroup label="'.$this->api->getLang($key).'">'."\n";
				foreach($opname as $key2=>$opname2)
				{
					if(is_array($opname2)) continue;
					$html.='<option value="'.$key2.'"'.(in_array($key2,$selected)?' selected':'').'>'.$this->api->getLang($opname2).'</option>'."\n";
				}
				$html.='</optgroup>';
			}
			else
			{
				$html.='<option value="'.$key.'"'.(in_array($key,$selected)?' selected':'').'>'.$this->api->getLang($opname).'</option>'."\n";
			}
		}
		$html.='</select>'."\n";
		return $html;
	}
	
	//Should be shortcut of text only field
	function text($name,$type='text',$value='',$attrs=array())
	{
		return $this->input($name,$type,$value,$attrs);
	}
	
	function input($name,$type='text',$value='',$attrs=array())
	{
		if($type=='token') return $this->token();
		
		if($type=='tel' || $type=='phone' || $type=='mobile')
		{
			isset($attrs['class'])? $attrs['class'].=' phone' : $attrs['class']='form-control phone';
			if(!isset($attrs['pattern'])) $attrs['pattern']='[0-9]{10,15}';
			if(!isset($attrs['minlength'])) $attrs['minlength']='10';
			if(!isset($attrs['maxlength']))  $attrs['maxlength']='15';
			$type='tel';
			$value=preg_replace("/[^0-9]/",'',$value);
		}
		else if($type=='datetime' || $type=='date' || $type=='time')
		{
			//$this->loadDateTime();
			isset($attrs['class'])? $attrs['class'].=' '.$type : $attrs['class']='form-control '.$type;
			$attrs['autocomplete']='off';
			$type='text';
		}
		else if($type=='color')
		{
			$this->loadColorPicker();
			isset($attrs['class'])? $attrs['class'].=' colorpicker' : $attrs['class']='form-control colorpicker';
			$type='text';
		}
		else if($type=='numeric')
		{
			$type='number';
		}
		
		if(is_array($value)) $value=reset($value);
		
		$html='';
		if($this->disabled && $type!='hidden')
		{
			$html.=$this->text($name,'hidden',$value,array('readonly'=>true));
			$attrs['disabled']='disabled';
			$name.='-off';
		}
		
		$atrstr=$this->parseAttributes($name,$attrs);		
		$html.='<input type="'.$type.'" name="'.$name.'" '.$atrstr.' value="'.$value.'" />';		
		return $html;
	}
		
	function radio($name,$options=array(),$selected=array(),$attrs=array())
	{
		if(!isset($attrs['class'])) $attrs['class']='form-control-radio';
		$selected=is_array($selected)?$selected:array($selected);
		$html='';
		foreach($options as $option=>$valName)
		{
			$radioAttrs=$attrs;
			if(in_array($option,$selected)) $radioAttrs['checked']='checked';
			$html.='<label>'.$this->text($name,'radio',$option, $radioAttrs).' '.$this->api->getLang($valName).'</label> ';
		}
		return $html;
	}
		
	function textarea($name,$value='',$attrs=array(),$htmlEditor=false)
	{
		if(!empty($htmlEditor))
		{
			$editor=is_bool($htmlEditor) || $htmlEditor==1 || $htmlEditor=='simple'?'htmleditorsimple':'htmleditor';
			$this->loadEditor();
			if(!isset($attrs['class'])) 
				$attrs['class']='form-control '.$editor;
			else
				$attrs['class'].=' '.$editor;
		}
		
		$html='';
		if($this->disabled)
		{
			$attrs['disabled']='disabled';
			$html.=$this->text($name,'hidden',$value,array('readonly'=>true));
			$name.='-off';
		}
		
		$atrstr=$this->parseAttributes($name,$attrs);
		$html.='<textarea name="'.$name.'" '.$atrstr.'>'.$value.'</textarea>';		
		return $html;
	}
}